@@ -18,6 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
 #include "gtkfilechooser.h"
 #include "gtkfilechooserprivate.h"
 #include "gtkfilesystem.h"
@@ -58,6 +59,24 @@ gtk_file_chooser_class_init (gpointer g_iface)
 {
   GType iface_type = G_TYPE_FROM_INTERFACE (g_iface);
 
+  /**
+   * GtkFileChooser::current-folder-changed
+   * @chooser: the object which received the signal.
+   *
+   * This signal is emitted when the current folder in a #GtkFileChooser
+   * changes.  This can happen due to the user performing some action that
+   * changes folders, such as selecting a bookmark or visiting a folder on the
+   * file list.  It can also happen as a result of calling a function to
+   * explicitly change the current folder in a file chooser.
+   *
+   * Normally you do not need to connect to this signal, unless you need to keep
+   * track of which folder a file chooser is showing.
+   *
+   * See also:  gtk_file_chooser_set_current_folder(),
+   * gtk_file_chooser_get_current_folder(),
+   * gtk_file_chooser_set_current_folder_uri(),
+   * gtk_file_chooser_get_current_folder_uri().
+   */
   g_signal_new ("current-folder-changed",
 		iface_type,
 		G_SIGNAL_RUN_LAST,
@@ -65,6 +84,26 @@ gtk_file_chooser_class_init (gpointer g_iface)
 		NULL, NULL,
 		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE, 0);
+
+  /**
+   * GtkFileChooser::selection-changed
+   * @chooser: the object which received the signal.
+   *
+   * This signal is emitted when there is a change in the set of selected files
+   * in a #GtkFileChooser.  This can happen when the user modifies the selection
+   * with the mouse or the keyboard, or when explicitly calling functions to
+   * change the selection.
+   *
+   * Normally you do not need to connect to this signal, as it is easier to wait
+   * for the file chooser to finish running, and then to get the list of
+   * selected files using the functions mentioned below.
+   *
+   * See also: gtk_file_chooser_select_filename(),
+   * gtk_file_chooser_unselect_filename(), gtk_file_chooser_get_filename(),
+   * gtk_file_chooser_get_filenames(), gtk_file_chooser_select_uri(),
+   * gtk_file_chooser_unselect_uri(), gtk_file_chooser_get_uri(),
+   * gtk_file_chooser_get_uris().
+   */
   g_signal_new ("selection-changed",
 		iface_type,
 		G_SIGNAL_RUN_LAST,
@@ -72,6 +111,33 @@ gtk_file_chooser_class_init (gpointer g_iface)
 		NULL, NULL,
 		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE, 0);
+
+  /**
+   * GtkFileChooser::update-preview
+   * @chooser: the object which received the signal.
+   *
+   * This signal is emitted when the preview in a file chooser should be
+   * regenerated.  For example, this can happen when the currently selected file
+   * changes.  You should use this signal if you want your file chooser to have
+   * a preview widget.
+   *
+   * Once you have installed a preview widget with
+   * gtk_file_chooser_set_preview_widget(), you should update it when this
+   * signal is emitted.  You can use the functions
+   * gtk_file_chooser_get_preview_filename() or
+   * gtk_file_chooser_get_preview_uri() to get the name of the file to preview.
+   * Your widget may not be able to preview all kinds of files; your callback
+   * must call gtk_file_chooser_set_preview_wiget_active() to inform the file
+   * chooser about whether the preview was generated successfully or not.
+   *
+   * Please see the example code in <xref linkend="gtkfilechooser-preview"/>.
+   *
+   * See also: gtk_file_chooser_set_preview_widget(),
+   * gtk_file_chooser_set_preview_widget_active(),
+   * gtk_file_chooser_set_use_preview_label(),
+   * gtk_file_chooser_get_preview_filename(),
+   * gtk_file_chooser_get_preview_uri().
+   */
   g_signal_new ("update-preview",
 		iface_type,
 		G_SIGNAL_RUN_LAST,
@@ -79,6 +145,23 @@ gtk_file_chooser_class_init (gpointer g_iface)
 		NULL, NULL,
 		g_cclosure_marshal_VOID__VOID,
 		G_TYPE_NONE, 0);
+
+  /**
+   * GtkFileChooser::file-activated
+   * @chooser: the object which received the signal.
+   *
+   * This signal is emitted when the user "activates" a file in the file
+   * chooser.  This can happen by double-clicking on a file in the file list, or
+   * by pressing <keycap>Enter</keycap>.
+   *
+   * Normally you do not need to connect to this signal.  It is used internally
+   * by #GtkFileChooserDialog to know when to activate the default button in the
+   * dialog.
+   *
+   * See also: gtk_file_chooser_get_filename(),
+   * gtk_file_chooser_get_filenames(), gtk_file_chooser_get_uri(),
+   * gtk_file_chooser_get_uris().
+   */
   g_signal_new ("file-activated",
 		iface_type,
 		G_SIGNAL_RUN_LAST,
@@ -359,16 +442,19 @@ gtk_file_chooser_get_filename (GtkFileChooser *chooser)
  * for the directory change. To pre-enter a filename for the user, as in
  * a save-as dialog, use gtk_file_chooser_set_current_name()
  *
+ * Return value: %TRUE if both the folder could be changed and the file was
+ * selected successfully, %FALSE otherwise.
+ *
  * Since: 2.4
  **/
-void
+gboolean
 gtk_file_chooser_set_filename (GtkFileChooser *chooser,
 			       const gchar    *filename)
 {
-  g_return_if_fail (GTK_IS_FILE_CHOOSER (chooser));
+  g_return_val_if_fail (GTK_IS_FILE_CHOOSER (chooser), FALSE);
 
   gtk_file_chooser_unselect_all (chooser);
-  gtk_file_chooser_select_filename (chooser, filename);
+  return gtk_file_chooser_select_filename (chooser, filename);
 }
 
 /**
@@ -380,26 +466,34 @@ gtk_file_chooser_set_filename (GtkFileChooser *chooser,
  * folder of @chooser, then the current folder of @chooser will
  * be changed to the folder containing @filename.
  *
+ * Return value: %TRUE if both the folder could be changed and the file was
+ * selected successfully, %FALSE otherwise.
+ *
  * Since: 2.4
  **/
-void
+gboolean
 gtk_file_chooser_select_filename (GtkFileChooser *chooser,
 				  const gchar    *filename)
 {
   GtkFileSystem *file_system;
   GtkFilePath *path;
+  gboolean result;
   
-  g_return_if_fail (GTK_IS_FILE_CHOOSER (chooser));
-  g_return_if_fail (filename != NULL);
+  g_return_val_if_fail (GTK_IS_FILE_CHOOSER (chooser), FALSE);
+  g_return_val_if_fail (filename != NULL, FALSE);
 
   file_system = _gtk_file_chooser_get_file_system (chooser);
 
   path = gtk_file_system_filename_to_path (file_system, filename);
   if (path)
     {
-      _gtk_file_chooser_select_path (chooser, path);
+      result = _gtk_file_chooser_select_path (chooser, path, NULL);
       gtk_file_path_free (path);
     }
+  else
+    result = FALSE;
+
+  return result;
 }
 
 /**
@@ -499,26 +593,34 @@ gtk_file_chooser_get_filenames (GtkFileChooser *chooser)
  * The user will be shown the full contents of the current folder,
  * plus user interface elements for navigating to other folders.
  *
+ * Return value: %TRUE if the folder could be changed successfully, %FALSE
+ * otherwise.
+ *
  * Since: 2.4
  **/
-void
+gboolean
 gtk_file_chooser_set_current_folder (GtkFileChooser *chooser,
 				     const gchar    *filename)
 {
   GtkFileSystem *file_system;
   GtkFilePath *path;
+  gboolean result;
   
-  g_return_if_fail (GTK_IS_FILE_CHOOSER (chooser));
-  g_return_if_fail (filename != NULL);
+  g_return_val_if_fail (GTK_IS_FILE_CHOOSER (chooser), FALSE);
+  g_return_val_if_fail (filename != NULL, FALSE);
 
   file_system = _gtk_file_chooser_get_file_system (chooser);
 
   path = gtk_file_system_filename_to_path (file_system, filename);
   if (path)
     {
-      _gtk_file_chooser_set_current_folder_path (chooser, path);
+      result = _gtk_file_chooser_set_current_folder_path (chooser, path, NULL);
       gtk_file_path_free (path);
     }
+  else
+    result = FALSE;
+
+  return result;
 }
 
 /**
@@ -628,16 +730,19 @@ gtk_file_chooser_get_uri (GtkFileChooser *chooser)
  * for the directory change. To pre-enter a filename for the user, as in
  * a save-as dialog, use gtk_file_chooser_set_current_name()
  *
+ * Return value: %TRUE if both the folder could be changed and the URI was
+ * selected successfully, %FALSE otherwise.
+ *
  * Since: 2.4
  **/
-void
+gboolean
 gtk_file_chooser_set_uri (GtkFileChooser *chooser,
 			  const char     *uri)
 {
-  g_return_if_fail (GTK_IS_FILE_CHOOSER (chooser));
+  g_return_val_if_fail (GTK_IS_FILE_CHOOSER (chooser), FALSE);
 
   gtk_file_chooser_unselect_all (chooser);
-  gtk_file_chooser_select_uri (chooser, uri);
+  return gtk_file_chooser_select_uri (chooser, uri);
 }
 
 /**
@@ -649,26 +754,34 @@ gtk_file_chooser_set_uri (GtkFileChooser *chooser,
  * file in the current folder of @chooser, then the current folder of
  * @chooser will be changed to the folder containing @filename.
  *
+ * Return value: %TRUE if both the folder could be changed and the URI was
+ * selected successfully, %FALSE otherwise.
+ *
  * Since: 2.4
  **/
-void
+gboolean
 gtk_file_chooser_select_uri (GtkFileChooser *chooser,
 			     const char     *uri)
 {
   GtkFileSystem *file_system;
   GtkFilePath *path;
+  gboolean result;
   
-  g_return_if_fail (GTK_IS_FILE_CHOOSER (chooser));
-  g_return_if_fail (uri != NULL);
+  g_return_val_if_fail (GTK_IS_FILE_CHOOSER (chooser), FALSE);
+  g_return_val_if_fail (uri != NULL, FALSE);
 
   file_system = _gtk_file_chooser_get_file_system (chooser);
 
   path = gtk_file_system_uri_to_path (file_system, uri);
   if (path)
     {
-      _gtk_file_chooser_select_path (chooser, path);
+      result = _gtk_file_chooser_select_path (chooser, path, NULL);
       gtk_file_path_free (path);
     }
+  else
+    result = FALSE;
+
+  return result;
 }
 
 /**
@@ -774,26 +887,34 @@ gtk_file_chooser_get_uris (GtkFileChooser *chooser)
  * The user will be shown the full contents of the current folder,
  * plus user interface elements for navigating to other folders.
  *
+ * Return value: %TRUE if the folder could be changed successfully, %FALSE
+ * otherwise.
+ *
  * Since: 2.4
  **/
-void
+gboolean
 gtk_file_chooser_set_current_folder_uri (GtkFileChooser *chooser,
 					 const gchar    *uri)
 {
   GtkFileSystem *file_system;
   GtkFilePath *path;
+  gboolean result;
   
-  g_return_if_fail (GTK_IS_FILE_CHOOSER (chooser));
-  g_return_if_fail (uri != NULL);
+  g_return_val_if_fail (GTK_IS_FILE_CHOOSER (chooser), FALSE);
+  g_return_val_if_fail (uri != NULL, FALSE);
 
   file_system = _gtk_file_chooser_get_file_system (chooser);
 
   path = gtk_file_system_uri_to_path (file_system, uri);
   if (path)
     {
-      _gtk_file_chooser_set_current_folder_path (chooser, path);
+      result = _gtk_file_chooser_set_current_folder_path (chooser, path, NULL);
       gtk_file_path_free (path);
     }
+  else
+    result = FALSE;
+
+  return result;
 }
 
 /**
@@ -830,20 +951,26 @@ gtk_file_chooser_get_current_folder_uri (GtkFileChooser *chooser)
  * _gtk_file_chooser_set_current_folder_path:
  * @chooser: a #GtkFileChooser
  * @path: the #GtkFilePath for the new folder
+ * @error: location to store error, or %NULL.
  * 
  * Sets the current folder for @chooser from a #GtkFilePath.
  * Internal function, see gtk_file_chooser_set_current_folder_uri().
  *
+ * Return value: %TRUE if the folder could be changed successfully, %FALSE
+ * otherwise.
+ *
  * Since: 2.4
  **/
-void
+gboolean
 _gtk_file_chooser_set_current_folder_path (GtkFileChooser    *chooser,
-					   const GtkFilePath *path)
+					   const GtkFilePath *path,
+					   GError           **error)
 {
-  g_return_if_fail (GTK_IS_FILE_CHOOSER (chooser));
-  g_return_if_fail (path != NULL);
+  g_return_val_if_fail (GTK_IS_FILE_CHOOSER (chooser), FALSE);
+  g_return_val_if_fail (path != NULL, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  GTK_FILE_CHOOSER_GET_IFACE (chooser)->set_current_folder (chooser, path);
+  return GTK_FILE_CHOOSER_GET_IFACE (chooser)->set_current_folder (chooser, path, error);
 }
 
 /**
@@ -870,19 +997,26 @@ _gtk_file_chooser_get_current_folder_path (GtkFileChooser *chooser)
  * _gtk_file_chooser_select_path:
  * @chooser: a #GtkFileChooser
  * @path: the path to select
+ * @error: location to store error, or %NULL
  * 
  * Selects the file referred to by @path. An internal function. See
  * _gtk_file_chooser_select_uri().
  *
+ * Return value: %TRUE if both the folder could be changed and the path was
+ * selected successfully, %FALSE otherwise.
+ *
  * Since: 2.4
  **/
-void
+gboolean
 _gtk_file_chooser_select_path (GtkFileChooser    *chooser,
-			       const GtkFilePath *path)
+			       const GtkFilePath *path,
+			       GError           **error)
 {
-  g_return_if_fail (GTK_IS_FILE_CHOOSER (chooser));
+  g_return_val_if_fail (GTK_IS_FILE_CHOOSER (chooser), FALSE);
+  g_return_val_if_fail (path != NULL, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  GTK_FILE_CHOOSER_GET_IFACE (chooser)->select_path (chooser, path);
+  return GTK_FILE_CHOOSER_GET_IFACE (chooser)->select_path (chooser, path, error);
 }
 
 /**
